@@ -1,12 +1,29 @@
-;; This is an example of how to use continuations with the Gtk event
-;; loop.  It implements a dialog box that looks like to the programmer
-;; like it was modal, and to the user like it was non-modal.  The
-;; function `yes-or-no?' that implements this dialog box only returns
-;; to the caller when the user has aswered the dialog.  The user
-;; however can pop up any number of these dialog boxes and answer them
-;; in any order he likes.  The main application stays alive as well.
+;; Copyright (C) 1998, 2002, 2006 Free Software Foundation, Inc.
+;;
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;; 
+;; This program is distributed in the hope that it will be useful, but
+;; WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; General Public License for more details.
+;; 
+;; You should have received a copy of the GNU General Public
+;; License along with this program; if not, write to the Free Software
+;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-(use-modules (gtk gtk))
+
+;; This is an example of how to use continuations with the Gtk event
+;; loop.  It implements a dialog box that looks to the programmer like
+;; it was modal, and to the user like it was non-modal.  The function
+;; `yes-or-no?' that implements this dialog box only returns to the
+;; caller when the user has aswered the dialog.  The user however can
+;; pop up any number of these dialog boxes and answer them in any
+;; order he likes.  The main application stays alive as well.
+
+(use-modules (gtk-2.0 gtk))
 
 ;; The callbacks that have been delayed
 
@@ -23,7 +40,7 @@
       (set! callbacks (cdr callbacks))
 	 (c)
 	 (event-loop)))
-   ((not (gtk-main-iteration))
+   ((not (zero? (gtk-main-iteration)))
     (event-loop))))
 
 ;; Connect to a signal and arrange for PROC to be consed onto
