@@ -18,5 +18,5 @@
 
 
 GdkPixbuf *gdk_pixbuf_new_from_xpm_data_interp (int len, char **data);
-gboolean gdk_pixbuf_loader_write_interp (GdkPixbufLoader *loader, SCM buf, SCM count);
+gboolean gdk_pixbuf_loader_write_interp (GdkPixbufLoader *loader, SCM buf, SCM count, GError **gerr);
 
