@@ -17,16 +17,30 @@
  * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#define GDK_ENABLE_BROKEN 1  /* ask for gdk_image_new_bitmap */
+
 #include <config.h>
 #include <libguile.h>
 #include <guile-gtk.h>
-#include <gtk-1.2/gdk/gdkprivate.h>
+
+#include <glib-object.h>
+#include <gdk/gdkx.h> /* for GDK_WINDOW_XID */
+
 #include "compat.h"
 #include "gtk-threads.h"
 #include <string.h>
 
 
 
+/* There's no g_object_get_type() (like say gtk_widget_get_type()) in gtk,
+   so create this little helper for use in the define-object of GObject in
+   gdk-2.0.defs.  */
+GType
+sgtk_g_object_get_type (void)
+{
+  return G_TYPE_OBJECT;
+}
+
 SCM
 gdk_selection_property_get_interp (GdkWindow *requestor,
 				   GdkAtom *prop_type, int *prop_format)
@@ -234,9 +248,9 @@ sgtk_gdk_gc_values_fill (char *func_name, int argnum,
       else if (scm_is_eq (key, kw_clip_mask))
 	{
           /* this is a bitmap, but the code and docs give it as GdkPixmap */
-          SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_pixmap_info),
+          SCM_ASSERT (sgtk_is_a_gtkobj (GDK_TYPE_PIXMAP, val),
                       val, argnum, func_name);
-          values->clip_mask = (GdkPixmap *) sgtk_scm2boxed (val);
+          values->clip_mask = (GdkPixmap *) sgtk_get_gtkobj (val);
 	  mask |= GDK_GC_CLIP_MASK;
 	}
       else if (scm_is_eq (key, kw_clip_x_origin))
@@ -301,9 +315,9 @@ sgtk_gdk_gc_values_fill (char *func_name, int argnum,
 	}
       else if (scm_is_eq (key, kw_stipple))
 	{
-          SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_pixmap_info),
+          SCM_ASSERT (sgtk_is_a_gtkobj (GDK_TYPE_PIXMAP, val),
                       val, argnum, func_name);
-          values->stipple = (GdkPixmap *) sgtk_scm2boxed (val);
+          values->stipple = (GdkPixmap *) sgtk_get_gtkobj (val);
 	  mask |= GDK_GC_STIPPLE;
 	}
       else if (scm_is_eq (key, kw_subwindow_mode))
@@ -315,9 +329,9 @@ sgtk_gdk_gc_values_fill (char *func_name, int argnum,
 	}
       else if (scm_is_eq (key, kw_tile))
 	{
-          SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_pixmap_info),
+          SCM_ASSERT (sgtk_is_a_gtkobj (GDK_TYPE_PIXMAP, val),
                       val, argnum, func_name);
-          values->tile = (GdkPixmap *) sgtk_scm2boxed (val);
+          values->tile = (GdkPixmap *) sgtk_get_gtkobj (val);
 	  mask |= GDK_GC_TILE;
 	}
       else if (scm_is_eq (key, kw_ts_x_origin))
@@ -403,9 +417,9 @@ gdk_window_new_interp (GdkWindow *parent, int width, int height,
 
       if (scm_is_eq (key, kw_colormap))
 	{
-          SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_colormap_info),
+          SCM_ASSERT (sgtk_is_a_gtkobj (GDK_TYPE_COLORMAP, val),
                       val, argnum, FUNC_NAME);
-          attr.colormap = (GdkColormap*) sgtk_scm2boxed (val);
+          attr.colormap = (GdkColormap*) sgtk_get_gtkobj (val);
 	  mask |= GDK_WA_COLORMAP;
 	}
       else if (scm_is_eq (key, kw_cursor))
@@ -429,9 +443,9 @@ gdk_window_new_interp (GdkWindow *parent, int width, int height,
 	}
       else if (scm_is_eq (key, kw_visual))
 	{
-          SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_visual_info),
+          SCM_ASSERT (sgtk_is_a_gtkobj (GDK_TYPE_VISUAL, val),
                       val, argnum, FUNC_NAME);
-          attr.visual = (GdkVisual*) sgtk_scm2boxed (val);
+          attr.visual = (GdkVisual*) sgtk_get_gtkobj (val);
 	  mask |= GDK_WA_VISUAL;
 	}
       else if (scm_is_eq (key, kw_wmclass))
@@ -487,7 +501,7 @@ gdk_window_destroy_interp (GdkWindow *window)
 static SCM
 _sgtk_helper_toscm_copy_GdkWindow (void *mem)
 {
-  return sgtk_boxed2scm ((*(GdkWindow**)mem), &sgtk_gdk_window_info, 1);
+  return sgtk_wrap_gtkobj ((GObject *) *(GdkWindow**) mem);
 }
 
 /* The list returned by gdk_window_get_children must be freed by the caller.
@@ -641,59 +655,7 @@ gdk_event_y (GdkEvent *event)
       return 0;
     }
 }
-#ifdef GTK_2_0
-#else
-gdouble
-gdk_event_pressure (GdkEvent *event)
-{
-  switch (event->any.type)
-    {
-    case GDK_MOTION_NOTIFY:
-      return event->motion.pressure;
-    case GDK_BUTTON_PRESS:
-    case GDK_BUTTON_RELEASE:
-    case GDK_2BUTTON_PRESS:
-    case GDK_3BUTTON_PRESS:
-      return event->button.pressure;
-    default:
-      return 0;
-    }
-}
 
-gdouble
-gdk_event_xtilt (GdkEvent *event)
-{
-  switch (event->any.type)
-    {
-    case GDK_MOTION_NOTIFY:
-      return event->motion.xtilt;
-    case GDK_BUTTON_PRESS:
-    case GDK_BUTTON_RELEASE:
-    case GDK_2BUTTON_PRESS:
-    case GDK_3BUTTON_PRESS:
-      return event->button.xtilt;
-    default:
-      return 0;
-    }
-}
-
-gdouble
-gdk_event_ytilt (GdkEvent *event)
-{
-  switch (event->any.type)
-    {
-    case GDK_MOTION_NOTIFY:
-      return event->motion.ytilt;
-    case GDK_BUTTON_PRESS:
-    case GDK_BUTTON_RELEASE:
-    case GDK_2BUTTON_PRESS:
-    case GDK_3BUTTON_PRESS:
-      return event->button.ytilt;
-    default:
-      return 0;
-    }
-}
-#endif
 gint
 gdk_event_button (GdkEvent *event)
 {
@@ -750,40 +712,40 @@ gdk_event_source (GdkEvent *event)
   switch (event->any.type)
     {
     case GDK_MOTION_NOTIFY:
-      return event->motion.source;
+      return event->motion.device->source;
     case GDK_BUTTON_PRESS:
     case GDK_BUTTON_RELEASE:
     case GDK_2BUTTON_PRESS:
     case GDK_3BUTTON_PRESS:
-      return event->button.source;
+      return event->button.device->source;
     case GDK_PROXIMITY_IN:
     case GDK_PROXIMITY_OUT:
-      return event->proximity.source;
+      return event->proximity.device->source;
     default:
       return GDK_SOURCE_MOUSE; /* XXX */
     }
 }
 
-
-guint32
-gdk_event_deviceid (GdkEvent *event)
-{
-  switch (event->any.type)
-    {
-    case GDK_MOTION_NOTIFY:
-      return event->motion.deviceid;
-    case GDK_BUTTON_PRESS:
-    case GDK_BUTTON_RELEASE:
-    case GDK_2BUTTON_PRESS:
-    case GDK_3BUTTON_PRESS:
-      return event->button.deviceid;
-    case GDK_PROXIMITY_IN:
-    case GDK_PROXIMITY_OUT:
-      return event->proximity.deviceid;
-    default:
-      return 0;
-    }
-}
+/* This can probably turn into gdk_event_device returning GdkDevice */
+/* guint32 */
+/* gdk_event_deviceid (GdkEvent *event) */
+/* { */
+/*   switch (event->any.type) */
+/*     { */
+/*     case GDK_MOTION_NOTIFY: */
+/*       return event->motion.deviceid; */
+/*     case GDK_BUTTON_PRESS: */
+/*     case GDK_BUTTON_RELEASE: */
+/*     case GDK_2BUTTON_PRESS: */
+/*     case GDK_3BUTTON_PRESS: */
+/*       return event->button.deviceid; */
+/*     case GDK_PROXIMITY_IN: */
+/*     case GDK_PROXIMITY_OUT: */
+/*       return event->proximity.deviceid; */
+/*     default: */
+/*       return 0; */
+/*     } */
+/* } */
 
 gdouble
 gdk_event_x_root (GdkEvent *event)
@@ -1100,23 +1062,6 @@ gdk_event_message (GdkEvent *event)
 }
 
 
-/* As per Gtk 2.0, made available in Gtk 1.2. */
-#if 0
-/* This implementation is bad because gdk_event_copy does a gdk_window_ref
-   on the window field, and that's invalid for a window of NULL.
-   gdk_event_free can cope with a NULL window, if such an event could be
-   created.  But let's not worry about event object creation for now.  */
-GdkEvent *
-gdk_event_new (GdkEventType type)
-{
-  GdkEvent event;
-  memset (&event, '\0', sizeof (event));
-  event.any.type = type;
-  return gdk_event_copy (&event);
-}
-#endif
-
-
 /* Doesn't work. */
 #if 0
 void
@@ -1135,16 +1080,24 @@ gdk_add_client_message_filter_interp (GdkAtom message_type, SCM filter_proc)
 }
 #endif
 
+/* A guile-gtk extension, retained for compatibility, but probably of
+   limited use.  Not sure what happens to GDK_WINDOW_XID on a non-X11
+   system.
+   In Gtk 2.0 there can be multiple displays and multiple leaders on one
+   display, so the idea here is just to give the default one.  */
 guint32
-gdk_get_leader_window_id ()
+gdk_get_leader_window_id (void)
 {
-  return (guint32) gdk_leader_window;
+  return GDK_WINDOW_XID
+    (gdk_display_get_default_group (gdk_display_get_default()));
 }
 
+/* A guile-gtk extension.  Not sure what happens to GDK_WINDOW_XID on a
+   non-X11 system.  */
 guint32
 gdk_window_get_id (GdkWindow *window)
 {
-  return (guint32) ((GdkWindowPrivate*)window)->xwindow;
+  return GDK_WINDOW_XID (window);
 }
 
 SCM_KEYWORD (kw_aspect,      "aspect");
@@ -1153,13 +1106,15 @@ SCM_KEYWORD (kw_max_size,    "max-size");
 SCM_KEYWORD (kw_min_size,    "min-size");
 SCM_KEYWORD (kw_resize_inc,  "resize-inc");
 SCM_KEYWORD (kw_pos,         "pos");
-#if 0  /* gtk 2 */
 SCM_KEYWORD (kw_user_pos,    "user-pos");
 SCM_KEYWORD (kw_user_size,   "user-size");
 SCM_KEYWORD (kw_win_gravity, "win-gravity");
-#endif
 
-/* argnum is always "2" in current uses, but gdk_window_constrain_size in
+/* Fill "geom" with keyword argument values taken "rest".  The return is a
+   GdkWindowHints mask showing which fields of "geom" have been set (plus
+   hint settings which are just flags, without data fields in GdkGeometry).
+
+   "argnum" is always 2 in current uses, but gdk_window_constrain_size in
    gtk 2 will be different.  */
 
 GdkWindowHints
@@ -1182,7 +1137,6 @@ sgtk_gdk_geometry_fill (const char *func_name, int argnum,
 	{
 	  mask |= GDK_HINT_POS;
 	}
-#if 0   /* gtk 2 */
       else if (scm_is_eq (key, kw_user_pos))
 	{
 	  mask |= GDK_HINT_USER_POS;
@@ -1191,7 +1145,6 @@ sgtk_gdk_geometry_fill (const char *func_name, int argnum,
 	{
 	  mask |= GDK_HINT_USER_SIZE;
 	}
-#endif
       else
         {
           if (! scm_is_pair (rest))
@@ -1201,16 +1154,14 @@ sgtk_gdk_geometry_fill (const char *func_name, int argnum,
           rest = SCM_CDR (rest);
           argnum++;
 
-#if 0  /* gtk 2 */
           if (scm_is_eq (key, kw_win_gravity))
             {
               geom->win_gravity
-                = sgtk_scm2enum (val, &sgtk_gdk_win_gravity_info,
+                = sgtk_scm2enum (val, &sgtk_gdk_gravity_info,
                                  argnum, func_name);
               mask |= GDK_HINT_WIN_GRAVITY;
             }
           else
-#endif
             {
               if (! scm_is_pair (rest))
                 scm_misc_error (func_name, "missing second argument to keyword ~A",
@@ -1422,11 +1373,9 @@ gdk_gc_get_values_interp (GdkGC *gc)
 		     sgtk_boxed2scm (values.font, &sgtk_gdk_font_info, TRUE),
 		     sgtk_enum2scm (values.function, &sgtk_gdk_function_info),
 		     sgtk_enum2scm (values.fill, &sgtk_gdk_fill_info),
-		     sgtk_boxed2scm (values.tile, &sgtk_gdk_window_info, TRUE),
-		     sgtk_boxed2scm (values.stipple,
-				     &sgtk_gdk_window_info, TRUE),
-		     sgtk_boxed2scm (values.clip_mask,
-				     &sgtk_gdk_window_info, TRUE),
+		     sgtk_wrap_gtkobj ((GObject *) values.tile),
+		     sgtk_wrap_gtkobj ((GObject *) values.stipple),
+		     sgtk_wrap_gtkobj ((GObject *) values.clip_mask),
 		     sgtk_enum2scm (values.subwindow_mode,
 				     &sgtk_gdk_subwindow_mode_info),
 		     scm_from_long (values.ts_x_origin),
@@ -1811,12 +1760,6 @@ gdk_color_black_interp (GdkColormap *colormap)
   return gdk_color_black (colormap, &color) ? gdk_color_copy (&color) : NULL;
 }
 
-guint
-gdk_color_hash_interp (GdkColor *color)
-{
-  return gdk_color_hash (color, NULL);
-}
-
 SCM
 gdk_query_depths_interp ()
 {
@@ -1848,7 +1791,7 @@ gdk_query_visual_types_interp ()
 static SCM
 _sgdk_helper_toscm_copy_GdkVisual (void *mem)
 {
-  return sgtk_boxed2scm (* (GdkVisual**) mem, &sgtk_gdk_visual_info, 1);
+  return sgtk_wrap_gtkobj ((GObject *) * (GdkVisual**) mem);
 }
 
 /* The list returned by gdk_list_visuals be freed by the caller.
@@ -1923,7 +1866,7 @@ gdk_property_get_interp (GdkWindow *window, GdkAtom property, GdkAtom type,
 
   return scm_take_locale_stringn ((char *) data, actual_length);
 }
-  
+
 
 
 void
@@ -1935,6 +1878,7 @@ sgtk_init_gdk_support ()
 #endif /* MKDEP */
 #endif /* SCM_MAGIC_SNARFER */
 
+  scm_c_define ("pango-scale", scm_from_int (PANGO_SCALE));
   scm_c_define ("gdk-priority-events", scm_from_int (GDK_PRIORITY_EVENTS));
   scm_c_define ("gdk-major-version", scm_from_int (gtk_major_version));
   scm_c_define ("gdk-minor-version", scm_from_int (gtk_minor_version));
