@@ -1,10 +1,24 @@
-(define-module (gtk-1.2 gtk)
-  :use-module (gtk-1.2 dynlink))
+;; Copyright (C) 2003, 2006 Free Software Foundation, Inc.
+;;
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;; 
+;; This program is distributed in the hope that it will be useful, but
+;; WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; General Public License for more details.
+;; 
+;; You should have received a copy of the GNU General Public
+;; License along with this program; if not, write to the Free Software
+;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-(define-public gtk-major-version 1)
-(define-public gtk-minor-version 2)
+(define-module (gtk-2.0 gtk)
+  :use-module (gtk-2.0 dynlink)
+  :use-module (gtk-2.0 gdk))
 
-(merge-compiled-code "sgtk_init_gtk_gtk_glue" "libguilegtk-1.2")
+(merge-compiled-code "sgtk_init_gtk_gtk_glue" "libguilegtk-2.0")
 
 (define-public (gtk-update)
   (cond ((> (gtk-events-pending) 0)
@@ -16,6 +30,12 @@
 	 (gtk-signal-connect toplevel "destroy" gtk-exit)
 	 (gtk-main))))
 
+(define-public (gtk-pixmap-new-from-file file parent)
+  (let ((mask (vector #f)))
+    (gtk-pixmap-new (gdk-pixmap-colormap-create-from-xpm 
+		      #f (gtk-widget-get-colormap parent) mask #f file)
+		    (vector-ref mask 0))))
+
 ;; Some aliases and quickies
 
 (define-public gtk-radio-menu-item-new 
@@ -27,7 +47,22 @@
 (define-public gtk-radio-button-new-with-label
   gtk-radio-button-new-with-label-from-widget)
 (define-public (gtk-idle-add proc)
-  (gtk-idle-add-full 0 proc))
+  (gtk-idle-add-full gtk-priority-default proc))
+(define-public gtk-idle-add-priority gtk-idle-add-full)
+
+;; GtkDialog
+
+(define-public (gtk-dialog-new-with-buttons title parent flags . text+ids)
+  (let ((dialog (%gtk-dialog-new-with-buttons title parent flags)))
+    (apply gtk-dialog-add-buttons dialog text+ids)
+    dialog))
+
+(define-public (gtk-dialog-add-buttons dialog . text+ids)
+  (or (null? text+ids)
+      (let ((text (car text+ids))
+	    (id   (cadr text+ids)))
+	(gtk-dialog-add-button dialog text id)
+	(apply gtk-dialog-add-buttons dialog (cddr text+ids)))))
 
 ;; The error reporter
 
