@@ -1,12 +1,25 @@
+/*
+ * Copyright (C) 1997, 1998, 2000, 2001, 2002, 2003, 2004, 2006 Free Software
+ * Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this software; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
 #include <config.h>
-#ifdef GTK_2_0
-#include <gtk-2.0/gtk/gtk.h>
-#include <gtk-2.0/gdk/gdkx.h>
-#else
-#include <gtk-1.2/gtk/gtk.h>
-#include <gtk-1.2/gdk/gdkx.h>
-#endif
-#include <guile-gtk.h>
+#include <gtk/gtk.h>
+#include "guile-gtk.h"
 
 /* This whole file is rated XXX. */
 
@@ -31,14 +44,6 @@ gtk_menu_popup_interp (GtkMenu *menu,
 		  NULL, NULL, button, activate_time);
 }
 
-GtkWidget*
-gtk_radio_menu_item_new_with_label_from_widget (GtkRadioMenuItem *group,
-						gchar            *label)
-{
-  GSList *g = group? gtk_radio_menu_item_group (group) : NULL;
-  return gtk_radio_menu_item_new_with_label (g, label);
-}
-
 GtkWidget*
 gtk_radio_menu_item_new_from_widget (GtkRadioMenuItem *group)
 {
@@ -46,29 +51,11 @@ gtk_radio_menu_item_new_from_widget (GtkRadioMenuItem *group)
   return gtk_radio_menu_item_new (g);
 }
 
-GtkWidget*
-gtk_pixmap_new_interp (gchar *file,
-		       GtkWidget *intended_parent)
-{
-  GtkStyle *style;
-  GdkPixmap *pixmap;
-  GdkBitmap *mask;
-
-  style = gtk_widget_get_style (intended_parent);
-  pixmap = gdk_pixmap_create_from_xpm (GDK_ROOT_PARENT(), &mask,
-				       &style->bg[GTK_STATE_NORMAL],
-				       file);
-  return gtk_pixmap_new (pixmap, mask);
-}
-
 GdkColor*
 gdk_color_parse_interp (char *spec)
 {
-  /* not reentrant */
-  static GdkColor color;
-  if (!gdk_color_parse (spec, &color))
-    return NULL;
-  return &color;
+  GdkColor color;
+  return gdk_color_parse (spec, &color) ? gdk_color_copy (&color) : NULL;
 }
 
 GdkColor*
@@ -77,14 +64,6 @@ gtk_style_get_white_interp (GtkStyle *style)
   return &style->white;
 }
 
-#ifndef HAVE_GTK_WIDGET_PEEK_COLORMAP
-GdkColormap *
-gtk_widget_peek_colormap ()
-{
-  return gtk_widget_get_default_colormap ();
-}
-#endif
-
 void
 gtk_list_append_item (GtkList *list, GtkListItem *item)
 {
@@ -101,15 +80,6 @@ gtk_list_prepend_item (GtkList *list, GtkListItem *item)
   gtk_list_prepend_items (list, items);
 }
 
-#ifndef HAVE_GTK_TYPE_GET_INFO
-gboolean
-gtk_type_get_info (GtkType type, GtkTypeInfo *info)
-{
-  g_warning("Your version of Gtk+ does not support gtk_type_get_info");
-  return FALSE;
-}
-#endif
-
 #ifndef HAVE_GTK_SIGNAL_SET_CLASS_FUNCTION_FULL
 void
 gtk_signal_set_class_function_full (GtkType            type,
@@ -127,8 +97,11 @@ gtk_signal_set_class_function_full (GtkType            type,
 void 
 gtk_color_selection_set_color_interp (GtkColorSelection *selection, GdkColor *color)
 {
-  gdouble vals[3];
+  gdouble vals[4];
   
+  /* get current opacity into vals[3], when in use, so we leave it unchanged */
+  gtk_color_selection_get_color (selection, vals);
+
   vals[0] = color->red / 65535.0; 
   vals[1] = color->green / 65535.0; 
   vals[2] = color->blue / 65535.0; 
@@ -140,13 +113,11 @@ gtk_color_selection_set_color_interp (GtkColorSelection *selection, GdkColor *co
 GdkColor *
 gtk_color_selection_get_color_interp (GtkColorSelection *selection)
 {
-  gdouble vals[3];
+  gdouble vals[4];
   GdkColor dummy, *color;
 
   gtk_color_selection_get_color (selection, vals);
 
-  /* XXX I don't know if this is a sensible way to obtain a new
-     GdkColor */
   color = gdk_color_copy (&dummy);
 
   /* Since this color is not part of a colormap, the pixel value is
