@@ -1,5 +1,5 @@
 /* Threading for guile-gtk
- * Copyright (C) 2000 Free Software Foundation
+ * Copyright (C) 2000, 2002, 2003, 2006 Free Software Foundation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * `gtk-threads-update' which makes the main loop wake up.
  */
 
-/* #define GUILE_GTKTHREADS_UPDATE */
+#define GUILE_GTKTHREADS_UPDATE 1
 
 #include <config.h>
 #include <libguile.h>
@@ -32,9 +32,19 @@
 
 #include "gtk-threads.h"
 
-#ifdef USE_THREADS
 
-extern int errno;
+/*---------------------------------------------------------------------------*/
+/* Guile 1.6 cooperative threads, ie ./configure --with-threads=coop
+ */
+
+#if USE_COOP_THREADS       /* Guile 1.6 */              \
+  || SCM_USE_COOP_THREADS  /* future Guile 1.8 maybe */ \
+  || SCM_USE_COPT_THREADS  /* future Guile 1.8 maybe */
+#define FOUND 1
+
+#if USE_COOP_THREADS && ! defined (SCM_USE_COOP_THREADS)
+#define SCM_USE_COOP_THREADS 1
+#endif
 
 #ifdef FD_SET
 #  define SELECT_MASK fd_set
@@ -167,10 +177,28 @@ static GMutex *
 g_mutex_new_guile_impl (void)
 {
   GMutex *result = (GMutex *) g_new (scm_t_mutex, 1);
+#ifdef SCM_MUTEX_INIT_TWO_ARGS
+  guile_check_for_error (scm_mutex_init ((scm_t_mutex *) result, 0));
+#else
   guile_check_for_error (scm_mutex_init ((scm_t_mutex *) result));
+#endif
   return result;
 }
 
+static void
+g_mutex_lock_guile_impl (GMutex * mutex)
+{
+  scm_t_mutex *m = (scm_t_mutex *) mutex;
+  scm_mutex_lock (m);
+}
+
+static void
+g_mutex_unlock_guile_impl (GMutex * mutex)
+{
+  scm_t_mutex *m = (scm_t_mutex *) mutex;
+  scm_mutex_unlock (m);
+}
+
 static void
 g_mutex_free_guile_impl (GMutex * mutex)
 {
@@ -192,7 +220,7 @@ g_mutex_trylock_guile_impl (GMutex * mutex)
 {
   int result;
 
-  result = coop_mutex_trylock ((scm_t_mutex *) mutex);
+  result = scm_mutex_trylock ((scm_t_mutex *) mutex);
 
   if (result == EBUSY)
     return FALSE;
@@ -285,18 +313,15 @@ g_private_get_guile_impl (GPrivate * private_key)
   return scm_getspecific (*(scm_t_key *) private_key);
 }
 
-#ifdef HAVE_THREAD_CREATE
 struct spawn_data {
   GThreadFunc func;
   gpointer arg;
-  gpointer thread;
 };
 
 static SCM
 spawn (void *arg)
 {
   struct spawn_data *data = (struct spawn_data *) arg;
-  * (coop_t **) data->thread = coop_global_curr;
   data->func (data->arg);
   return SCM_UNSPECIFIED;
 }
@@ -308,22 +333,42 @@ g_thread_create_guile_impl (GThreadFunc thread_func,
 			    gboolean joinable,
 			    gboolean bound,
 			    GThreadPriority priority,
-			    gpointer thread)
+			    gpointer thread,
+                            GError **error)
 {
   struct spawn_data data;
+  SCM t;
   data.func = thread_func;
   data.arg = arg;
-  data.thread = thread;
-  scm_spawn_thread (spawn, &data, scm_handle_by_message_noexit, 0);
+  t = scm_spawn_thread (spawn, &data, scm_handle_by_message_noexit, 0);
+  * (SCM *) thread = t;
 }
 
 static void
 g_thread_join_guile_impl (gpointer thread)
 {
-  coop_join (* (coop_t **) thread);
+#ifdef SCM_USE_COOP_THREADS
+  coop_join ((coop_t *) SCM_THREAD_DATA ((SCM) thread));
+#endif
+#ifdef SCM_USE_COPT_THREADS
+  scm_join_thread ((SCM) thread);
+#endif
 }
 
+#ifdef SCM_USE_COOP_THREADS
 extern void coop_abort (void);
+#endif
+
+static void
+g_thread_exit_guile_impl (void)
+{
+#ifdef SCM_USE_COOP_THREADS
+  coop_abort ();
+#else
+  fprintf (stderr, "g_thread_exit_guile_impl not implemented\n");
+  abort ();
+#endif
+}
 
 static void
 g_thread_set_priority_guile_impl (gpointer thread, GThreadPriority priority)
@@ -333,41 +378,47 @@ g_thread_set_priority_guile_impl (gpointer thread, GThreadPriority priority)
 static void
 g_thread_self_guile_impl (gpointer thread)
 {
-  * (coop_t **) thread = coop_global_curr;
+#ifdef SCM_USE_COOP_THREADS
+  scm_root_state *rs = coop_global_curr->data;
+  SCM self = rs->handle;
+  * (SCM *) thread = self;
+#else
+#ifdef SCM_USE_COPT_THREADS
+  * (SCM *) thread = cur_thread;
+#else
+#error Oops, no g_thread_self_guile_impl code
+#endif
+#endif
 }
-#endif /* HAVE_THREAD_CREATE */
 
-static GThreadFunctions g_guile_thread_functions_for_glib =
+static GThreadFunctions g_thread_functions =
 {
   g_mutex_new_guile_impl,
-  (void (*)(GMutex *)) scm_mutex_lock,
+  g_mutex_lock_guile_impl,
   g_mutex_trylock_guile_impl,
-  (void (*)(GMutex *)) scm_mutex_unlock,
+  g_mutex_unlock_guile_impl,
   g_mutex_free_guile_impl,
   g_cond_new_guile_impl,
-  (void (*)(GCond *)) scm_cond_signal,
-  (void (*)(GCond *)) scm_cond_broadcast,
-  (void (*)(GCond *, GMutex *)) scm_cond_wait,
+  scm_cond_signal, /* cond_signal */
+  scm_cond_broadcast, /* cond_broadcast */
+  scm_cond_wait, /* cond_wait */
   g_cond_timed_wait_guile_impl,
   g_cond_free_guile_impl,
   g_private_new_guile_impl,
   g_private_get_guile_impl,
-  g_private_set_guile_impl
-#ifdef HAVE_THREAD_CREATE
-  ,
+  g_private_set_guile_impl,
   g_thread_create_guile_impl,
-  coop_yield,
+  (void (*)(void)) scm_yield,
   g_thread_join_guile_impl,
-  coop_abort,
+  g_thread_exit_guile_impl,
   g_thread_set_priority_guile_impl,
   g_thread_self_guile_impl
-#endif /* HAVE_THREAD_CREATE */
 };
 
 void
 sgtk_init_threads ()
 {
-  g_thread_init (&g_guile_thread_functions_for_glib);
+  g_thread_init (&g_thread_functions);
   g_main_set_poll_func (g_poll);
 #ifdef GUILE_GTKTHREADS_UPDATE
   if (pipe (wake_up_pipe) < 0)
@@ -376,11 +427,48 @@ sgtk_init_threads ()
 
   wake_up_rec.fd = wake_up_pipe[0];
   wake_up_rec.events = G_IO_IN;
-  g_main_add_poll (&wake_up_rec, 0);
+  g_main_context_add_poll (g_main_context_default(), &wake_up_rec, 0);
 #endif
 }
 
-#else /* !USE_THREADS */
+#endif /* coop threads */
+
+
+/*---------------------------------------------------------------------------*/
+/* Guile 1.8 posix pthread threads, ie ./configure --with-threads=pthreads
+ */
+
+#if SCM_USE_PTHREAD_THREADS   /* Guile 1.8 */
+#define FOUND 1
+
+SCM
+sgtk_threads_update ()
+{
+  return SCM_UNSPECIFIED;
+}
+
+void
+sgtk_init_threads ()
+{
+  /* NULL here asks for the default threading, which is posix pthreads on a
+     GNU/Linux system */
+  g_thread_init (NULL);
+}
+
+#endif /* USE_PTHREAD_THREADS */
+
+
+/*---------------------------------------------------------------------------*/
+/* Guile 1.6 or 1.8 null threads, ie ./configure --with-threads=no (or null)
+
+   "! USE_THREADS" is the test in 1.6, but check also the version number,
+   because USE_THREADS is gone altogether in 1.8 too (or at least exists
+   only under SCM_ENABLE_DEPRECATED=1)  */
+
+#if (SCM_MAJOR_VERSION == 1 && SCM_MINOR_VERSION == 6 &&        \
+     ! USE_THREADS)         /* Guile 1.6 */                     \
+  || SCM_USE_NULL_THREADS   /* Guile 1.8 */
+#define FOUND 1
 
 SCM
 sgtk_threads_update ()
@@ -393,4 +481,11 @@ sgtk_init_threads ()
 {
 }
 
-#endif /* !USE_THREADS */
+#endif /* USE_NULL_THREADS */
+
+
+
+/*---------------------------------------------------------------------------*/
+#if ! FOUND
+#error "Oops, Guile threading system recognised"
+#endif
