@@ -26,11 +26,21 @@ fi
 ACDIR=`which $ACLOCAL`
 ACDIR=`dirname $ACDIR`
 ACDIR=`dirname $ACDIR`/share/aclocal
-GTDIR=`gtk-config --prefix`/share/aclocal
 GUDIR=`guile-config info prefix`/share/aclocal
 
-AFLAGS="-I macros "
-for each in $GUDIR $GTDIR $G2DIR $G1DIR $ACDIR ; do
+# We used to look in wherever gtk-config (the gtk 1.2 program) said Gtk is
+# installed, but no longer since Gtk 2 uses pkg-config.
+#
+# GTDIR=`gtk-config --prefix`/share/aclocal
+
+# Back in Gtk 1.2 we had copies of some .m4 macro files in a macros
+# directory in the source tree, but no longer.  (aclocal is upset if you
+# give it a non-existant directory in a -I, so this has to be disabled.)
+#
+# AFLAGS="-I macros "
+
+AFLAGS=""
+for each in $GUDIR $G2DIR $G1DIR $ACDIR ; do
     if test -d "$each"  ; then 
 	AFLAGS="-I $each $AFLAGS"
 	break
