@@ -1,7 +1,8 @@
 #include "guile-gtk.h"
 
 int
-main ()
+main (void)
 {
   sgtk_init ();
+  return 0;
 }
