@@ -21,8 +21,8 @@
       (exit 77)))
 
 (use-modules (srfi srfi-1)
-	     (gtk-1.2 gdk)
-	     (gtk-1.2 gtk))
+	     (gtk-2.0 gdk)
+	     (gtk-2.0 gtk))
 
 (define sample-win     (gdk-window-new #f 100 100 '() 'input-output 'toplevel))
 (define sample-gc       (gdk-gc-new sample-win))
@@ -31,6 +31,7 @@
 ;; the first-ever load of a given font name ties up some memory in a hash
 ;; table, or something, so make that happen here
 (define sample-font     (gdk-font-load "fixed"))
+(define sample-bitmap   (gdk-pixmap-new #f 1 1 1))
 
 ;; don't put "(ash 1 256)" inside a `malloced-steady' because there was a
 ;; bug in guile 1.6.7 and earlier where the `bytes-malloced' wasn't
@@ -65,7 +66,191 @@
   (and (exact-integer? x)
        (positive? x)))
 
+;;
+;; gdk-drawable-get-image
+;;
 
+(malloced-steady
+ (lambda ()
+   (let* ((image (gdk-drawable-get-image sample-bitmap 0 0 1 1))
+	  (want  #t)
+	  ;; just let malloced-steady see our ref counting on the returned
+	  ;; image is right
+	  (got   (gdk-image? image)))
+     (test "gdk-drawable-get-image" image want got))))
+
+;;
+;; gdk-drawable-copy-to-image
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((image (gdk-drawable-copy-to-image sample-bitmap #f 0 0 0 0 1 1))
+	  (want  #t)
+	  ;; just let malloced-steady see our ref counting on the returned
+	  ;; image is right
+	  (got   (gdk-image? image)))
+     (test "gdk-drawable-copy-to-image, new" image want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((orig-image (gdk-drawable-get-image sample-bitmap 0 0 1 1))
+	  (new-image  (gdk-drawable-copy-to-image sample-bitmap orig-image
+						  0 0 0 0 1 1))
+	  (want       #t)
+	  ;; just let malloced-steady see our ref counting on the returned
+	  ;; image is right
+	  (got        (gdk-image? new-image)))
+     (test "gdk-drawable-copy-to-image, existing" orig-image want got))))
+
+;;
+;; GdkFunction
+;;
+
+(let* ((bitmap (gdk-pixmap-new #f 1 1 1))
+       (gc0    (gdk-gc-new bitmap))
+       (gc1    (gdk-gc-new bitmap))
+       (gc     (gdk-gc-new bitmap))
+       (color  (gdk-color-new)))
+
+  (define (my-gdk-get-point drawable x y)
+    (let ((image (gdk-drawable-get-image drawable x y 1 1)))
+      (gdk-image-get-pixel image 0 0)))
+
+  (define (attempt function src dst want)
+
+    (gdk-color-set-pixel color src)
+    (gdk-gc-set-foreground gc color)
+    (gdk-gc-set-function gc function)
+
+    (gdk-draw-point bitmap (if (= 0 dst) gc0 gc1) 0 0)
+
+    (gdk-draw-point bitmap gc 0 0)
+
+    (malloced-steady
+     (lambda ()
+       (let ((got (my-gdk-get-point bitmap 0 0)))
+	 (test "GdkFunction using gdk-draw-point" (list function src dst)
+	       want got)))))
+
+  (gdk-color-set-pixel color 0)
+  (gdk-gc-set-foreground gc0 color)
+  (gdk-gc-set-function gc0 'copy)
+
+  (gdk-color-set-pixel color 1)
+  (gdk-gc-set-foreground gc1 color)
+  (gdk-gc-set-function gc1 'copy)
+
+  (attempt 'copy 0 0 0)
+  (attempt 'copy 0 1 0)
+  (attempt 'copy 1 0 1)
+  (attempt 'copy 1 1 1)
+
+  (attempt 'invert 0 0 1)
+  (attempt 'invert 0 1 0)
+  (attempt 'invert 1 0 1)
+  (attempt 'invert 1 1 0)
+
+  (attempt 'xor 0 0 0)
+  (attempt 'xor 0 1 1)
+  (attempt 'xor 1 0 1)
+  (attempt 'xor 1 1 0)
+
+  (attempt 'clear 0 0 0)
+  (attempt 'clear 0 1 0)
+  (attempt 'clear 1 0 0)
+  (attempt 'clear 1 1 0)
+
+  (attempt 'and 0 0 0)
+  (attempt 'and 0 1 0)
+  (attempt 'and 1 0 0)
+  (attempt 'and 1 1 1)
+
+  (attempt 'and-reverse 0 0 0)
+  (attempt 'and-reverse 0 1 0)
+  (attempt 'and-reverse 1 0 1)
+  (attempt 'and-reverse 1 1 0)
+
+  (attempt 'and-invert 0 0 0)
+  (attempt 'and-invert 0 1 1)
+  (attempt 'and-invert 1 0 0)
+  (attempt 'and-invert 1 1 0)
+
+  (attempt 'noop 0 0 0)
+  (attempt 'noop 0 1 1)
+  (attempt 'noop 1 0 0)
+  (attempt 'noop 1 1 1)
+
+  (attempt 'or 0 0 0)
+  (attempt 'or 0 1 1)
+  (attempt 'or 1 0 1)
+  (attempt 'or 1 1 1)
+
+  (attempt 'equiv 0 0 1)
+  (attempt 'equiv 0 1 0)
+  (attempt 'equiv 1 0 0)
+  (attempt 'equiv 1 1 1)
+
+  (attempt 'or-reverse 0 0 1)
+  (attempt 'or-reverse 0 1 0)
+  (attempt 'or-reverse 1 0 1)
+  (attempt 'or-reverse 1 1 1)
+
+  (attempt 'copy-invert 0 0 1)
+  (attempt 'copy-invert 0 1 1)
+  (attempt 'copy-invert 1 0 0)
+  (attempt 'copy-invert 1 1 0)
+
+  (attempt 'or-invert 0 0 1)
+  (attempt 'or-invert 0 1 1)
+  (attempt 'or-invert 1 0 0)
+  (attempt 'or-invert 1 1 1)
+
+  (attempt 'nand 0 0 1)
+  (attempt 'nand 0 1 1)
+  (attempt 'nand 1 0 1)
+  (attempt 'nand 1 1 0)
+
+  (attempt 'nor 0 0 1)
+  (attempt 'nor 0 1 0)
+  (attempt 'nor 1 0 0)
+  (attempt 'nor 1 1 0)
+
+  (attempt 'set 0 0 1)
+  (attempt 'set 0 1 1)
+  (attempt 'set 1 0 1)
+  (attempt 'set 1 1 1))
+
+;;
+;; gdk-char-width
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((char  #\m)
+	  (want  #t)
+	  (got   (number? (gdk-char-width sample-font char))))
+     (test "gdk-char-width" char want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((char  "m")  ;; bad
+	  (want  #f)
+	  (got   (false-if-exception
+		  (begin
+		    (gdk-char-width sample-font char)
+		    #t))))
+     (test "gdk-char-width" char want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((char  65)  ;; bad
+	  (want  #f)
+	  (got   (false-if-exception
+		  (begin
+		    (gdk-char-width sample-font char)
+		    #t))))
+     (test "gdk-char-width" char want got))))
 
 ;;
 ;; gdk-atom-intern
@@ -87,13 +272,15 @@
 	  (got    (false-if-exception (gdk-atom-intern name notnew))))
      (test "gdk-atom-intern" (list name notnew) want got))))
 
-(malloced-steady
- (lambda ()
-   (let* ((name   "ATOM_NOT_OTHERWISE_EXISTING")
-	  (notnew #t)
-	  (want   #f)
-	  (got    (gdk-atom-intern name notnew)))
-     (test "gdk-atom-intern" (list name notnew) want got))))
+;; Gtk 2.0 ignores the notnew flag, atom always created.
+;;
+;; (malloced-steady
+;;  (lambda ()
+;;    (let* ((name   "ATOM_NOT_OTHERWISE_EXISTING")
+;; 	  (notnew #t)
+;; 	  (want   #f)
+;; 	  (got    (gdk-atom-intern name notnew)))
+;;      (test "gdk-atom-intern" (list name notnew) want got))))
 
 (malloced-steady
  (lambda ()
@@ -234,6 +421,34 @@
 		    #t))))
      (test "gdk-char-width" char want got))))
 
+;;
+;; gdk-color-black
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((color (gdk-color-black sample-colormap))
+	  (want  #t)
+	  (got   (or (eq? #f color)
+		     (eqv? 0 (gdk-color-red   color))
+		     (eqv? 0 (gdk-color-green color))
+		     (eqv? 0 (gdk-color-blue  color)))))
+     (test "gdk-color-black" color want got))))
+
+;;
+;; gdk-color-white
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((color (gdk-color-white sample-colormap))
+	  (want  #t)
+	  (got   (or (eq? #f color)
+		     (eqv? 65535 (gdk-color-red   color))
+		     (eqv? 65535 (gdk-color-green color))
+		     (eqv? 65535 (gdk-color-blue  color)))))
+     (test "gdk-color-white" color want got))))
+
 ;;
 ;; gdk-colormap-alloc-colors
 ;;
@@ -276,6 +491,25 @@
 
       (display "skip gdk-colormap-alloc-colors: no pseudo-color visual\n")))
 
+;;
+;; gdk-colormap-query-color
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((white       (gdk-color-white sample-colormap))
+	  (white-pixel (gdk-color-pixel white))
+	  (result      (gdk-color-new))
+	  (want        #t)
+	  (got (begin
+		 (gdk-colormap-query-color sample-colormap white-pixel
+					   result)
+		 (and
+		  (eqv? (gdk-color-red   white) (gdk-color-red   result))
+		  (eqv? (gdk-color-blue  white) (gdk-color-blue  result))
+		  (eqv? (gdk-color-green white) (gdk-color-green result))))))
+     (test "gdk-colormap-query-color" white want got))))
+
 ;;
 ;; gdk-drag-begin
 ;;
@@ -485,6 +719,13 @@
 	  (got  (->bool (apply gdk-gc-new-with-values args))))
      (test "gdk-gc-new-with-values" args want got))))
 
+(malloced-steady
+ (lambda ()
+   (let* ((args (list sample-win #:font "fixed"))
+	  (want #t)
+	  (got  (->bool (apply gdk-gc-new-with-values args))))
+     (test "gdk-gc-new-with-values" args want got))))
+
 
 ;;
 ;; gdk-gc-set-dashes
@@ -590,6 +831,28 @@
 		   #t)))
      (test "gdk-gc-set-foreground" color want got))))
 
+;;
+;; gdk-get-leader-window-id
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((id   (gdk-get-leader-window-id))
+	  (want #t)
+	  (got  (exact-positive-integer? id)))
+     (test "gdk-get-leader-window-id" #f want got))))
+
+;;
+;; gdk-image-new
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  (list 'fastest (gdk-visual-get-system) 10 10))
+	  (image (apply gdk-image-new args)))
+     ;; only checking no memory leak
+     (test "gdk-image-new" args #t #t))))
+
 ;;
 ;; gdk-image-new-bitmap
 ;;
@@ -614,7 +877,7 @@
  (lambda ()
    (let* ((want #t)
 	  (got  (and (exact-nonnegative-integer? gdk-major-version)
-		     (>= gdk-major-version 1))))
+		     (>= gdk-major-version 2))))
      (test "gdk-major-version" gdk-major-version want got))))
 
 ;;
@@ -847,21 +1110,21 @@
 (malloced-steady  ;; with nothing ever requested
  (lambda ()
    (let* ((win    sample-win)
-	  (want   '(#f #f 0))
+	  (want   '(#f NONE 0))
 	  (got    (gdk-selection-property-get win)))
      (test "gdk-selection-property-get" sample-win want got))))
 
 (malloced-steady  ;; on a destroyed window
  (lambda ()
    (let* ((win    (gdk-window-new #f 100 100 '() 'input-output 'toplevel))
-	  (want   '(#f #f 0))
+	  (want   '(#f NONE 0))
 	  (got    (begin
 		    (gdk-window-destroy win)
 		    (gdk-selection-property-get win))))
      ;; final free()s only take place under the main loop, so need this to
      ;; satisfy `malloced-steady
      (gtk-main-iteration-do #f)
-     (test "gdk-selection-property-get" win want got))))
+     (test "gdk-selection-property-get" (list "destroyed win" win) want got))))
 
 ;;
 ;; gdk-text-extents
@@ -1058,6 +1321,17 @@
 	    ;; proximity-out-mask  ;; does nothing in gdk 1.2
 	    substructure-mask))
 
+;;
+;; gdk-window-get-id
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((id   (gdk-window-get-id sample-win))
+	  (want #t)
+	  (got  (exact-positive-integer? id)))
+     (test "gdk-window-get-id" sample-win want got))))
+
 ;;
 ;; gdk-window-new
 ;;
