@@ -20,8 +20,8 @@
       (display "Cannot run gtk tests without an X display\n")
       (exit 77)))
 
-(use-modules (gtk-1.2 gtk)
-	     (gtk-1.2 gdk))
+(use-modules (gtk-2.0 gtk)
+	     (gtk-2.0 gdk))
 
 ;; the first-ever load of a given font name ties up some memory in a hash
 ;; table, or something, so make that happen here
@@ -55,6 +55,7 @@
   (and (exact-integer? x)
        (positive? x)))
 
+
 ;;
 ;; gtk-adjustment-set-value
 ;;
@@ -108,12 +109,26 @@
  (lambda ()
    (let* ((child (gtk-label-new "foo"))
 	  (want  child)
-	  (bin   (gtk-widget-new 'GtkBin))
+	  (frame (gtk-frame-new "title"))
 	  (got   (begin
-		   (gtk-container-add bin child)
-		   (gtk-bin-child bin))))
+		   (gtk-container-add frame child)
+		   (gtk-bin-child frame))))
      (test "gtk-bin-child" #f want got))))
 
+;;
+;; gtk-bin-get-child
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((child (gtk-label-new "foo"))
+	  (want  child)
+	  (frame (gtk-frame-new "title"))
+	  (got   (begin
+		   (gtk-container-add frame child)
+		   (gtk-bin-get-child frame))))
+     (test "gtk-bin-get-child" #f want got))))
+
 ;;
 ;; gtk-box-query-child-packing
 ;;
@@ -130,6 +145,30 @@
 		   (gtk-box-query-child-packing box child))))
      (test "gtk-box-query-child-packing" pack want got))))
 
+;;
+;; gtk-button-get-label
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((button (gtk-button-new-with-label "foo"))
+	  (want   "foo")
+	  (got    (gtk-button-get-label button)))
+     (test "gtk-button-get-label" button want got))))
+
+;;
+;; gtk-button-set-label
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((button (gtk-button-new-with-label "foo"))
+	  (want   "bar")
+	  (got    (begin
+		    (gtk-button-set-label button "bar")
+		    (gtk-button-get-label button))))
+     (test "gtk-button-get-label" button want got))))
+
 ;;
 ;; gtk-check-version
 ;;
@@ -229,10 +268,10 @@
      (gtk-main-iteration-do #f)
      (test "gtk-combo-set-popdown-strings" strings want got))))
 
-(malloced-steady   ;; empty list
+(malloced-steady   ;; empty list is allowed
  (lambda ()
    (let* ((strings '())
-	  (want    #f)
+	  (want    '())
 	  (combo   (gtk-combo-new))
 	  (got     (false-if-exception
 		    (begin
@@ -243,10 +282,10 @@
      (gtk-main-iteration-do #f)
      (test "gtk-combo-set-popdown-strings" strings want got))))
 
-(malloced-steady   ;; empty vector
+(malloced-steady   ;; empty vector is allowed
  (lambda ()
    (let* ((strings (vector))
-	  (want    #f)
+	  (want    '())
 	  (combo   (gtk-combo-new))
 	  (got     (false-if-exception
 		    (begin
@@ -316,10 +355,10 @@
 ;; gtk-fixed-put
 ;;
 
-(malloced-steady     ;; y too big for 16 bits
+(malloced-steady     ;; y too big
  (lambda ()
    (let* ((x     123)
-	  (y     32768)
+	  (y     sample-bignum)
 	  (fixed (gtk-fixed-new))
 	  (child (gtk-label-new "foo"))
 	  (want  #f)
@@ -483,7 +522,7 @@
  (lambda ()
    (let* ((want #t)
 	  (got  (and (exact-nonnegative-integer? gtk-major-version)
-		     (>= gtk-major-version 1))))
+		     (>= gtk-major-version 2))))
      (test "gtk-major-version" #f want got))))
 
 ;;
@@ -516,6 +555,36 @@
 	  (got  (exact-nonnegative-integer? gtk-binary-age)))
      (test "gtk-binary-age" #f want got))))
 
+;;
+;; gtk-idle-remove
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((id   (gtk-idle-add noop))
+	  (want #t)
+	  (got  (begin
+		  (gtk-idle-remove id)
+		  #t)))
+     (test "gtk-idle-remove" id want got))))
+
+;;
+;; gtk-input-remove
+;;
+
+;; port buffer space miscounted in malloc stats in guile 1.6, so don't have
+;; the pipe inside malloced-steady
+;;
+(let ((port  (cdr (pipe))))
+  (malloced-steady
+   (lambda ()
+     (let* ((id   (gtk-input-add port '(write) noop))
+	    (want #t)
+	    (got  (begin
+		    (gtk-input-remove id)
+		    #t)))
+       (test "gtk-input-remove" id want got)))))
+
 ;;
 ;; gtk-interface-age
 ;;
@@ -539,7 +608,14 @@
 
 (malloced-steady
  (lambda ()
-   (let* ((args  (list 'NoSuchType))
+   (let* ((args (list 'NoSuchType))
+	  (want #f)
+	  (got  (false-if-exception (apply gtk-object-new args))))
+     (test "gtk-object-new" args want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  (list 'GtkObject))  ;; cannot be instantiated
 	  (want  #f)
 	  (got   (false-if-exception (apply gtk-object-new args))))
      (test "gtk-object-new" args want got))))
@@ -557,19 +633,15 @@
      (test "gtk-object-new label" args want got))))
 
 ;;
-;; gtk-container-query-child-args
-;;
-;; Would like to look at the actual return value contents, but there's no
-;; easy way at the moment to get GtkType objects to compare with what's
-;; returned.
+;; gtk-item-factory-path-from-widget
 ;;
 
 (malloced-steady
  (lambda ()
-   (let* ((type 'GtkTable)
-	  (want #t)
-	  (got  (list? (gtk-container-query-child-args type))))
-     (test "gtk-container-query-child-args" type want got))))
+   (let* ((widget (gtk-label-new "foo"))
+	  (want   #f)
+	  (got    (gtk-item-factory-path-from-widget widget)))
+     (test "gtk-item-factory-path-from-widget" widget want got))))
 
 ;;
 ;; gtk-object-set string
@@ -709,22 +781,36 @@
      ;; only checking for no memory leak
      (test "gtk-signal-disconnect" button #t #t))))
 
+;;
+;; gtk-timeout-remove
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((id   (gtk-timeout-add 100000 noop))
+	  (want #t)
+	  (got  (begin
+		  (gtk-timeout-remove id)
+		  #t)))
+     (test "gtk-timeout-remove" id want got))))
+
+
 ;;
 ;; gtk-toolbar-set-style
 ;;
 
 (malloced-steady
  (lambda ()
-   (let* ((toolbar (gtk-toolbar-new 'horizontal 'icons))
+   (let* ((toolbar (gtk-toolbar-new))
 	  (style   'both)
 	  (want    style)
 	  (got     (begin
 		     (gtk-toolbar-set-style toolbar style)
-		     (gtk-object-get toolbar #:toolbar_style))))
+		     (gtk-toolbar-get-style toolbar))))
      (test "gtk-toolbar-set-style" (list toolbar style) want got))))
 
 ;;
-;; gtk-widget-size-request
+;; gtk-widget-size-request string
 ;;
 
 (malloced-steady
@@ -741,8 +827,8 @@
 ;; signal call font
 ;;
 
-(gtk-signal-new-generic "test-call-font" '() 'GtkObject 'void '(GdkFont))
-(let ((obj (gtk-object-new 'GtkObject))
+(gtk-signal-new-generic "test-call-font" '() 'GtkLabel 'void '(GdkFont))
+(let ((obj (gtk-object-new 'GtkLabel))
       (ret #f))
   (gtk-signal-connect obj "test-call-font" noop)
 
@@ -768,9 +854,9 @@
 ;; signal call string+int
 ;;
 
-(gtk-signal-new-generic "test-call-string+int" '() 'GtkObject
-			'void '(GtkString gint))
-(let ((obj (gtk-object-new 'GtkObject))
+(gtk-signal-new-generic "test-call-string+int" '() 'GtkLabel
+			'void '(gchararray gint))
+(let ((obj (gtk-object-new 'GtkLabel))
       (ret #f))
   (gtk-signal-connect obj "test-call-string+int" noop)
 
@@ -800,10 +886,11 @@
 ;; Somehow the return is always GTK_TYPE_NONE so you don't get back a
 ;; string.  The tests here are in readiness if that's fixed.
 
-(gtk-signal-new-generic "test-call-return-string" '() 'GtkObject
-			'GtkString '())
-(let ((obj (gtk-object-new 'GtkObject))
+(gtk-signal-new-generic "test-call-return-string" '() 'GtkLabel
+			'gchararray '())
+(let ((obj (gtk-object-new 'GtkLabel))
       (ret #f))
+
   (gtk-signal-connect obj "test-call-return-string"
     (lambda ()
       ret))
@@ -838,25 +925,67 @@
 		    #t)))
        (test "gtk-signal-emit call return string" str want got))))
 
-  (malloced-steady  ;; bad type
-   (lambda ()
-     (let* ((str  999)
-	    (want #t)
-	    (got  (begin
-		    (set! ret str)
-		    (gtk-signal-emit obj "test-call-return-string")
-		    #t)))
-       (test "gtk-signal-emit call return string" str want got))))
+;; FIXME: Don't want to print error message.
 
-  (malloced-steady  ;; bomb with \0 in string
+;;   (malloced-steady  ;; bad type
+;;    (lambda ()
+;;      (let* ((str  999)
+;; 	    (want #t)
+;; 	    (got  (begin
+;; 		    (set! ret str)
+;; 		    (gtk-signal-emit obj "test-call-return-string")
+;; 		    #t)))
+;;        (test "gtk-signal-emit call return string" str want got))))
+
+;;   (malloced-steady  ;; bomb with \0 in string
+;;    (lambda ()
+;;      (let* ((str  (string #\nul))
+;; 	    (want #t)
+;; 	    (got  (begin
+;; 		    (set! ret str)
+;; 		    (gtk-signal-emit obj "test-call-return-string")
+;; 		    #t)))
+;;        (test "gtk-signal-emit call return string" str want got))))
+  )
+
+;;
+;; signal call GType (new in glib 2.12)
+;;
+
+(gtk-signal-new-generic "test-call-gtype" '() 'GtkLabel 'void '(GType))
+(let ((obj          (gtk-object-new 'GtkLabel))
+      (arg-received #f))
+  (gtk-signal-connect obj "test-call-gtype"
+    (lambda (arg)
+      (set! arg-received arg)))
+
+  (malloced-steady  ;; gtype
    (lambda ()
-     (let* ((str  (string #\nul))
-	    (want #t)
+     (let* ((type (gtk-type-from-name "GdkWindow"))
+	    (want type)
 	    (got  (begin
-		    (set! ret str)
-		    (gtk-signal-emit obj "test-call-return-string")
-		    #t)))
-       (test "gtk-signal-emit call return string" str want got)))))
+		    (gtk-signal-emit obj "test-call-gtype" type)
+		    arg-received)))
+       (test "gtk-signal-emit call gtype" type want got))))
+
+  (malloced-steady  ;; symbol in
+   (lambda ()
+     (let* ((type 'GdkWindow)
+	    (want (gtk-type-from-name (symbol->string type)))
+	    (got  (begin
+		    (gtk-signal-emit obj "test-call-gtype" type)
+		    arg-received)))
+       (test "gtk-signal-emit call gtype" type want got))))
+
+  (malloced-steady  ;; number - bogus
+   (lambda ()
+     (let* ((type 999)
+	    (want #f)
+	    (got  (false-if-exception
+		   (begin
+		     (gtk-signal-emit obj "test-call-gtype" type)
+		     #t))))
+       (test "gtk-signal-emit call gtype" type want got)))))
 
 
 (tests-end)
